@@ -2,13 +2,14 @@
 
 import pandas as pd
 from urllib.parse import urljoin
-from .genesig import Regulon, GeneSignature, openfile
+from ctxcore.genesig import Regulon, GeneSignature, openfile
 from .math import masked_rho4pairs
 from itertools import chain
 import numpy as np
 from functools import partial
 from typing import Sequence, Type
 from yaml import load, dump
+
 try:
     from yaml import CLoader as Loader, CDumper as Dumper
 except ImportError:
@@ -26,11 +27,12 @@ COLUMN_NAME_ORTHOLOGOUS_IDENTITY = 'OrthologousIdentity'
 COLUMN_NAME_ANNOTATION = 'Annotation'
 
 
-def load_motif_annotations(fname: str,
-                           column_names=('#motif_id', 'gene_name',
-                                         'motif_similarity_qvalue', 'orthologous_identity', 'description'),
-                           motif_similarity_fdr: float = 0.001,
-                           orthologous_identity_threshold: float = 0.0) -> pd.DataFrame:
+def load_motif_annotations(
+    fname: str,
+    column_names=('#motif_id', 'gene_name', 'motif_similarity_qvalue', 'orthologous_identity', 'description'),
+    motif_similarity_fdr: float = 0.001,
+    orthologous_identity_threshold: float = 0.0,
+) -> pd.DataFrame:
     """
     Load motif annotations from a motif2TF snapshot.
 
@@ -43,13 +45,20 @@ def load_motif_annotations(fname: str,
     """
     # Create a MultiIndex for the index combining unique gene name and motif ID. This should facilitate
     # later merging.
-    df = pd.read_csv(fname, sep='\t', index_col=[1,0], usecols=column_names)
+    df = pd.read_csv(fname, sep='\t', index_col=[1, 0], usecols=column_names)
     df.index.names = [COLUMN_NAME_TF, COLUMN_NAME_MOTIF_ID]
-    df.rename(columns={'motif_similarity_qvalue': COLUMN_NAME_MOTIF_SIMILARITY_QVALUE,
-                       'orthologous_identity': COLUMN_NAME_ORTHOLOGOUS_IDENTITY,
-                       'description': COLUMN_NAME_ANNOTATION }, inplace=True)
-    df = df[(df[COLUMN_NAME_MOTIF_SIMILARITY_QVALUE] <= motif_similarity_fdr) &
-            (df[COLUMN_NAME_ORTHOLOGOUS_IDENTITY] >= orthologous_identity_threshold)]
+    df.rename(
+        columns={
+            'motif_similarity_qvalue': COLUMN_NAME_MOTIF_SIMILARITY_QVALUE,
+            'orthologous_identity': COLUMN_NAME_ORTHOLOGOUS_IDENTITY,
+            'description': COLUMN_NAME_ANNOTATION,
+        },
+        inplace=True,
+    )
+    df = df[
+        (df[COLUMN_NAME_MOTIF_SIMILARITY_QVALUE] <= motif_similarity_fdr)
+        & (df[COLUMN_NAME_ORTHOLOGOUS_IDENTITY] >= orthologous_identity_threshold)
+    ]
     return df
 
 
@@ -80,8 +89,9 @@ def _create_idx_pairs(adjacencies: pd.DataFrame, exp_mtx: pd.DataFrame) -> np.nd
     return np.array([[symbol2idx[s1], symbol2idx[s2]] for s1, s2 in zip(adjacencies.TF, adjacencies.target)])
 
 
-def add_correlation(adjacencies: pd.DataFrame, ex_mtx: pd.DataFrame,
-                    rho_threshold=RHO_THRESHOLD, mask_dropouts=False) -> pd.DataFrame:
+def add_correlation(
+    adjacencies: pd.DataFrame, ex_mtx: pd.DataFrame, rho_threshold=RHO_THRESHOLD, mask_dropouts=False
+) -> pd.DataFrame:
     """
     Add correlation in expression levels between target and factor.
 
@@ -131,17 +141,19 @@ def add_correlation(adjacencies: pd.DataFrame, ex_mtx: pd.DataFrame,
     else:
         genes = list(set(adjacencies[COLUMN_NAME_TF]).union(set(adjacencies[COLUMN_NAME_TARGET])))
         ex_mtx = ex_mtx[ex_mtx.columns[ex_mtx.columns.isin(genes)]]
-        corr_mtx = pd.DataFrame(index=ex_mtx.columns, columns=ex_mtx.columns,
-                                data=np.corrcoef(ex_mtx.values.T))
+        corr_mtx = pd.DataFrame(index=ex_mtx.columns, columns=ex_mtx.columns, data=np.corrcoef(ex_mtx.values.T))
         rhos = np.array([corr_mtx[s2][s1] for s1, s2 in zip(adjacencies.TF, adjacencies.target)])
 
     regulations = (rhos > rho_threshold).astype(int) - (rhos < -rho_threshold).astype(int)
-    return pd.DataFrame(data={
-        COLUMN_NAME_TF: adjacencies[COLUMN_NAME_TF].values,
-        COLUMN_NAME_TARGET: adjacencies[COLUMN_NAME_TARGET].values,
-        COLUMN_NAME_WEIGHT: adjacencies[COLUMN_NAME_WEIGHT].values,
-        COLUMN_NAME_REGULATION: regulations,
-        COLUMN_NAME_CORRELATION: rhos})
+    return pd.DataFrame(
+        data={
+            COLUMN_NAME_TF: adjacencies[COLUMN_NAME_TF].values,
+            COLUMN_NAME_TARGET: adjacencies[COLUMN_NAME_TARGET].values,
+            COLUMN_NAME_WEIGHT: adjacencies[COLUMN_NAME_WEIGHT].values,
+            COLUMN_NAME_REGULATION: regulations,
+            COLUMN_NAME_CORRELATION: rhos,
+        }
+    )
 
 
 def modules4thr(adjacencies, threshold, context=frozenset(), pattern="weight>{:.3f}"):
@@ -158,7 +170,8 @@ def modules4thr(adjacencies, threshold, context=frozenset(), pattern="weight>{:.
                 context=frozenset([pattern.format(threshold)]).union(context),
                 transcription_factor=tf_name,
                 gene2weight=list(zip(df_grp[COLUMN_NAME_TARGET].values, df_grp[COLUMN_NAME_WEIGHT].values)),
-                gene2occurrence=[])
+                gene2occurrence=[],
+            )
 
 
 def modules4top_targets(adjacencies, n, context=frozenset()):
@@ -176,7 +189,8 @@ def modules4top_targets(adjacencies, n, context=frozenset()):
                 context=frozenset(["top{}".format(n)]).union(context),
                 transcription_factor=tf_name,
                 gene2weight=list(zip(module[COLUMN_NAME_TARGET].values, module[COLUMN_NAME_WEIGHT].values)),
-                gene2occurrence=[])
+                gene2occurrence=[],
+            )
 
 
 def modules4top_factors(adjacencies, n, context=frozenset()):
@@ -194,27 +208,30 @@ def modules4top_factors(adjacencies, n, context=frozenset()):
                 context=frozenset(["top{}perTarget".format(n)]).union(context),
                 transcription_factor=tf_name,
                 gene2weight=list(zip(df_grp[COLUMN_NAME_TARGET].values, df_grp[COLUMN_NAME_WEIGHT].values)),
-                gene2occurrence=[])
+                gene2occurrence=[],
+            )
 
 
 ACTIVATING_MODULE = "activating"
 REPRESSING_MODULE = "repressing"
 
 
-def modules_from_adjacencies(adjacencies: pd.DataFrame,
-                             ex_mtx: pd.DataFrame,
-                        thresholds=(0.75, 0.90),
-                        top_n_targets=(50,),
-                        top_n_regulators=(5,10,50),
-                        min_genes=20,
-                        absolute_thresholds=False,
-                        rho_dichotomize=True,
-                        keep_only_activating=True,
-                        rho_threshold=RHO_THRESHOLD,
-                        rho_mask_dropouts=False) -> Sequence[Regulon]:
+def modules_from_adjacencies(
+    adjacencies: pd.DataFrame,
+    ex_mtx: pd.DataFrame,
+    thresholds=(0.75, 0.90),
+    top_n_targets=(50,),
+    top_n_regulators=(5, 10, 50),
+    min_genes=20,
+    absolute_thresholds=False,
+    rho_dichotomize=True,
+    keep_only_activating=True,
+    rho_threshold=RHO_THRESHOLD,
+    rho_mask_dropouts=False,
+) -> Sequence[Regulon]:
     """
     Create modules from a dataframe containing weighted adjacencies between a TF and its target genes.
-    
+
     :param adjacencies: The dataframe with the TF-target links. This dataframe should have the following columns:
         :py:const:`pyscenic.utils.COLUMN_NAME_TF`, :py:const:`pyscenic.utils.COLUMN_NAME_TARGET` and :py:const:`pyscenic.utils.COLUMN_NAME_WEIGHT` .
     :param ex_mtx: The expression matrix (n_cells x n_genes).
@@ -242,17 +259,25 @@ def modules_from_adjacencies(adjacencies: pd.DataFrame,
     # To make the pySCENIC code more robust to the selection of the network inference method in the first step of
     # the pipeline, it is better to use percentiles instead of absolute values for the weight thresholds.
     if not absolute_thresholds:
+
         def iter_modules(adjc, context):
-            yield from chain(chain.from_iterable(
-                                    modules4thr(adjc, thr, context, pattern="weight>{}%".format(frac*100))
-                                    for thr, frac in zip(list(adjacencies[COLUMN_NAME_WEIGHT].quantile(thresholds)), thresholds)),
-                             chain.from_iterable(modules4top_targets(adjc, n, context) for n in top_n_targets),
-                             chain.from_iterable(modules4top_factors(adjc, n, context) for n in top_n_regulators))
+            yield from chain(
+                chain.from_iterable(
+                    modules4thr(adjc, thr, context, pattern="weight>{}%".format(frac * 100))
+                    for thr, frac in zip(list(adjacencies[COLUMN_NAME_WEIGHT].quantile(thresholds)), thresholds)
+                ),
+                chain.from_iterable(modules4top_targets(adjc, n, context) for n in top_n_targets),
+                chain.from_iterable(modules4top_factors(adjc, n, context) for n in top_n_regulators),
+            )
+
     else:
+
         def iter_modules(adjc, context):
-            yield from chain(chain.from_iterable(modules4thr(adjc, thr, context) for thr in thresholds),
-                         chain.from_iterable(modules4top_targets(adjc, n, context) for n in top_n_targets),
-                         chain.from_iterable(modules4top_factors(adjc, n, context) for n in top_n_regulators))
+            yield from chain(
+                chain.from_iterable(modules4thr(adjc, thr, context) for thr in thresholds),
+                chain.from_iterable(modules4top_targets(adjc, n, context) for n in top_n_targets),
+                chain.from_iterable(modules4top_factors(adjc, n, context) for n in top_n_regulators),
+            )
 
     if not rho_dichotomize:
         # Do not differentiate between activating and repressing modules.
@@ -265,11 +290,18 @@ def modules_from_adjacencies(adjacencies: pd.DataFrame,
             # Add correlation column and create two disjoint set of adjacencies.
             LOGGER.info("Calculating Pearson correlations.")
             # test for genes present in the adjacencies but not present in the expression matrix:
-            unique_adj_genes = set(adjacencies[COLUMN_NAME_TF]).union(set(adjacencies[COLUMN_NAME_TARGET])) - set(ex_mtx.columns)
-            assert len(unique_adj_genes)==0, f"Found {len(unique_adj_genes)} genes present in the network (adjacencies) output, but missing from the expression matrix. Is this a different gene expression matrix?"
-            LOGGER.warn(f"Note on correlation calculation: the default behaviour for calculating the correlations has changed after pySCENIC verion 0.9.16. Previously, the default was to calculate the correlation between a TF and target gene using only cells with non-zero expression values (mask_dropouts=True). The current default is now to use all cells to match the behavior of the R verision of SCENIC. The original settings can be retained by setting 'rho_mask_dropouts=True' in the modules_from_adjacencies function, or '--mask_dropouts' from the CLI.\n\tDropout masking is currently set to [{rho_mask_dropouts}].")
-            adjacencies = add_correlation(adjacencies, ex_mtx,
-                                      rho_threshold=rho_threshold, mask_dropouts=rho_mask_dropouts)
+            unique_adj_genes = set(adjacencies[COLUMN_NAME_TF]).union(set(adjacencies[COLUMN_NAME_TARGET])) - set(
+                ex_mtx.columns
+            )
+            assert (
+                len(unique_adj_genes) == 0
+            ), f"Found {len(unique_adj_genes)} genes present in the network (adjacencies) output, but missing from the expression matrix. Is this a different gene expression matrix?"
+            LOGGER.warn(
+                f"Note on correlation calculation: the default behaviour for calculating the correlations has changed after pySCENIC verion 0.9.16. Previously, the default was to calculate the correlation between a TF and target gene using only cells with non-zero expression values (mask_dropouts=True). The current default is now to use all cells to match the behavior of the R verision of SCENIC. The original settings can be retained by setting 'rho_mask_dropouts=True' in the modules_from_adjacencies function, or '--mask_dropouts' from the CLI.\n\tDropout masking is currently set to [{rho_mask_dropouts}]."
+            )
+            adjacencies = add_correlation(
+                adjacencies, ex_mtx, rho_threshold=rho_threshold, mask_dropouts=rho_mask_dropouts
+            )
         else:
             LOGGER.info("Using existing Pearson correlations from the adjacencies file.")
 
@@ -278,8 +310,10 @@ def modules_from_adjacencies(adjacencies: pd.DataFrame,
             modules_iter = iter_modules(activating_modules, frozenset([ACTIVATING_MODULE]))
         else:
             repressing_modules = adjacencies[adjacencies[COLUMN_NAME_REGULATION] < 0.0]
-            modules_iter = chain(iter_modules(activating_modules, frozenset([ACTIVATING_MODULE])),
-                             iter_modules(repressing_modules, frozenset([REPRESSING_MODULE])))
+            modules_iter = chain(
+                iter_modules(activating_modules, frozenset([ACTIVATING_MODULE])),
+                iter_modules(repressing_modules, frozenset([REPRESSING_MODULE])),
+            )
 
     # Derive modules for these adjacencies.
     # + Add the transcription factor to the module.
@@ -288,8 +322,10 @@ def modules_from_adjacencies(adjacencies: pd.DataFrame,
     #    repressing modules]
     # + Filter for minimum number of genes.
     LOGGER.info("Creating modules.")
+
     def add_tf(module):
         return module.add(module.transcription_factor)
+
     return list(filter(lambda m: len(m) >= min_genes, map(add_tf, modules_iter)))
 
 
@@ -323,7 +359,9 @@ def add_motif_url(df: pd.DataFrame, base_url: str):
     :param base_url:
     :return:
     """
-    df[("Enrichment", COLUMN_NAME_MOTIF_URL)] = list(map(partial(urljoin, base=base_url), df.index.get_level_values(COLUMN_NAME_MOTIF_ID)))
+    df[("Enrichment", COLUMN_NAME_MOTIF_URL)] = list(
+        map(partial(urljoin, base_url), df.index.get_level_values(COLUMN_NAME_MOTIF_ID))
+    )
     return df
 
 
@@ -336,9 +374,7 @@ def load_motifs(fname: str, sep: str = ',') -> pd.DataFrame:
     """
     from .transform import COLUMN_NAME_TARGET_GENES, COLUMN_NAME_CONTEXT
 
-    df = pd.read_csv(fname, sep=sep, index_col=[0,1], header=[0,1], skipinitialspace=True)
+    df = pd.read_csv(fname, sep=sep, index_col=[0, 1], header=[0, 1], skipinitialspace=True)
     df[('Enrichment', COLUMN_NAME_CONTEXT)] = df[('Enrichment', COLUMN_NAME_CONTEXT)].apply(lambda s: eval(s))
     df[('Enrichment', COLUMN_NAME_TARGET_GENES)] = df[('Enrichment', COLUMN_NAME_TARGET_GENES)].apply(lambda s: eval(s))
     return df
-
-
